# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2024 CERN.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Banner Resource Configuration."""

import marshmallow as ma
from flask_resources import JSONDeserializer, RequestBodyParser
from invenio_records_resources.resources import (
    RecordResourceConfig,
    SearchRequestArgsSchema,
)


class BannerServerSearchRequestArgsSchema(SearchRequestArgsSchema):
    """Banner request parameters."""

    sort_direction = ma.fields.Str()


class BannerResourceConfig(RecordResourceConfig):
    """Banner resource config."""

    # Blueprint configuration
    blueprint_name = "banners"
    url_prefix = "/banners"
    routes = {
        "item": "/<banner_id>",
        "list": "/",
    }

    request_view_args = {
        "banner_id": ma.fields.String(),
    }

    request_extra_args = {
        "active": ma.fields.Boolean(),
        "url_path": ma.fields.String(),
    }

    request_search_args = BannerServerSearchRequestArgsSchema

    request_body_parsers = {"application/json": RequestBodyParser(JSONDeserializer())}
    default_content_type = "application/json"

    response_handlers = {
        "application/vnd.inveniordm.v1+json": RecordResourceConfig.response_handlers[
            "application/json"
        ],
        **RecordResourceConfig.response_handlers,
    }



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2023 CERN.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Banners schema."""

from datetime import datetime, timezone

from invenio_records_resources.services.records.schema import BaseRecordSchema
from marshmallow import fields, pre_load
from marshmallow_utils.fields import SanitizedHTML, TZDateTime


class BannerSchema(BaseRecordSchema):
    """Schema for banners."""

    message = SanitizedHTML(required=True)
    url_path = fields.String(allow_none=True)
    category = fields.String(required=True, metadata={"default": "info"})
    start_datetime = fields.DateTime(
        required=True,
        metadata={"default": datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S")},
    )
    end_datetime = fields.DateTime(allow_none=True)
    active = fields.Boolean(required=True, metadata={"default": True})
    created = TZDateTime(timezone=timezone.utc, format="iso", dump_only=True)
    updated = TZDateTime(timezone=timezone.utc, format="iso", dump_only=True)

    @pre_load
    def change_none_to_string(self, data, **kwargs):
        """Fix for empty strings not in line with allow_none=True."""
        for field in data:
            if field == "end_datetime" or field == "category":
                if data[field] == "":
                    data[field] = None
        return data



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 CERN.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Utils."""

from flask import request

from invenio_banners.services.config import BannerModel


def get_active_banners_for_request():
    """Get active banner for the current URL path request."""
    url_path = request.path
    return BannerModel.get_active(url_path)


def style_category(category):
    """Return predefined Semantic-UI classes for each banner category."""
    style_class = "{}"
    if category == "warning":
        style_class = style_class.format("warning")
    elif category == "other":
        style_class = style_class.format("grey")
    else:
        style_class = style_class.format("info")
    return style_class



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Proxies for accessing the current Banners extension."""

from flask import current_app
from werkzeug.local import LocalProxy

current_banners = LocalProxy(lambda: current_app.extensions["invenio-banners"])
"""Proxy for the instantiated Banners extension."""

current_banners_service = LocalProxy(
    lambda: current_app.extensions["invenio-banners"].banners_service
)
"""Proxy for the currently instantiated banners service."""



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 CERN.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Module tests."""

from flask import Flask

from invenio_banners import InvenioBanners


def test_version():
    """Test version import."""
    from invenio_banners import __version__

    assert __version__


def test_init():
    """Test extension initialization."""
    app = Flask("testapp")
    InvenioBanners(app)
    assert "invenio-banners" in app.extensions

    app = Flask("testapp")
    ext = InvenioBanners()
    assert "invenio-banners" not in app.extensions
    ext.init_app(app)
    assert "invenio-banners" in app.extensions



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2023 CERN.
# Copyright (C) 2024 Graz University of Technology.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Service results."""

from invenio_records_resources.services.records.results import RecordItem, RecordList

try:
    # flask_sqlalchemy<3.0.0
    from flask_sqlalchemy import Pagination
except ImportError:
    # flask_sqlalchemy>=3.0.0
    from flask_sqlalchemy.pagination import Pagination


class BannerItem(RecordItem):
    """Single banner result."""

    def __init__(
        self,
        service,
        identity,
        banner,
        links_tpl=None,
        errors=None,
        schema=None,
    ):
        """Constructor."""
        super().__init__(service, identity, banner, errors, links_tpl, schema)

    @property
    def data(self):
        """Property to get the banner."""
        if self._data:
            return self._data

        self._data = self._schema.dump(
            self._obj,
            context={
                "identity": self._identity,
                "record": self._record,
            },
        )

        if self._links_tpl:
            self._data["links"] = self.links

        return self._data


class BannerList(RecordList):
    """List of banner results."""

    def __init__(
        self,
        service,
        identity,
        banners,
        params=None,
        links_tpl=None,
        links_item_tpl=None,
        schema=None,
    ):
        """Constructor."""
        super().__init__(
            service, identity, banners, params, links_tpl, links_item_tpl, schema
        )

    @property
    def hits(self):
        """Iterator over the hits."""
        for record in self.banners_result():
            # Project the record
            projection = self._schema.dump(
                record,
                context=dict(
                    identity=self._identity,
                    record=record,
                ),
            )

            if self._links_item_tpl:
                projection["links"] = self._links_item_tpl.expand(
                    self._identity, record
                )

            yield projection

    def to_dict(self):
        """Return result as a dictionary."""
        res = {
            "hits": {
                "hits": list(self.hits),
                "total": self.total,
            }
        }

        if self._params:
            if self._links_tpl:
                res["links"] = self._links_tpl.expand(self._identity, self.pagination)

        return res

    @property
    def total(self):
        """Get total number of banners."""
        return (
            self._results.total
            if isinstance(self._results, Pagination)
            else len(self._results)
        )

    def banners_result(self):
        """Get iterable banners list."""
        return (
            self._results.items
            if isinstance(self._results, Pagination)
            else self._results
        )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 CERN.
# Copyright (C) 2024 Graz University of Technology.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Errors."""

import marshmallow as ma
from flask_resources import HTTPJSONException, create_error_handler
from invenio_records_resources.errors import validation_error_to_list_errors

from ..services.errors import BannerNotExistsError


class HTTPJSONValidationException(HTTPJSONException):
    """HTTP exception serializing to JSON and reflecting Marshmallow errors."""

    description = "A validation error occurred."

    def __init__(self, exception):
        """Constructor."""
        super().__init__(code=400, errors=validation_error_to_list_errors(exception))


class ErrorHandlersMixin:
    """Mixin to define error handlers."""

    error_handlers = {
        BannerNotExistsError: create_error_handler(
            lambda e: HTTPJSONException(
                code=404,
                description=e.description,
            )
        ),
        ma.ValidationError: create_error_handler(
            lambda e: HTTPJSONValidationException(e)
        ),
    }



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 CERN.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio administration banners view module."""

from invenio_administration.views.base import (
    AdminResourceCreateView,
    AdminResourceDetailView,
    AdminResourceEditView,
    AdminResourceListView,
)
from invenio_i18n import lazy_gettext as _


class BannerListView(AdminResourceListView):
    """Search admin view."""

    api_endpoint = "/banners"
    name = "banners"
    resource_config = "banners_resource"
    title = "Banners"
    menu_label = "Banners"
    category = _("Site management")
    pid_path = "id"
    icon = "newspaper"

    display_search = True
    display_delete = True
    display_create = True
    display_edit = True

    item_field_list = {
        "id": {"text": _("Id"), "order": 1, "width": 1},
        "start_datetime": {"text": _("Start time (UTC)"), "order": 2, "width": 2},
        "end_datetime": {"text": _("End time (UTC)"), "order": 3, "width": 2},
        "message": {"text": _("Message"), "order": 4, "width": 7},
        "active": {"text": _("Active"), "order": 5, "width": 1},
        "url_path": {"text": _("URL path"), "order": 6, "width": 2},
        "category": {"text": _("Category"), "order": 7, "width": 1},
    }

    create_view_name = "banner_create"

    search_config_name = "BANNERS_SEARCH"
    search_sort_config_name = "BANNERS_SORT_OPTIONS"


common_form_fields = {
    "start_datetime": {
        "order": 1,
        "text": _("Start time"),
        "description": _(
            "Date/time to make the banner active. "
            "Input format: yyyy-mm-dd hh:mm:ss. "
            "Set to future date/time to delay the banner. "
            "Note: specify time in UTC time standard."
        ),
        "placeholder": _("YYYY-MM-DD hh:mm:ss"),
    },
    "end_datetime": {
        "order": 2,
        "text": _("End time"),
        "description": _(
            "Date/time to make the banner inactive. "
            "Input format: yyyy-mm-dd hh:mm:ss.  An empty value makes "
            "the banner active until manually disabled via the active flag. "
            "Note: specify time in UTC time standard."
        ),
        "placeholder": _("YYYY-MM-DD hh:mm:ss"),
    },
    "message": {
        "order": 3,
        "text": _("Message"),
        "description": _(
            "Message to be displayed on the banner. HTML format is supported."
        ),
        "rows": 10,
    },
    "url_path": {
        "order": 4,
        "text": _("URL path"),
        "description": _(
            "URL path prefix (including the first /) to define where "
            "the message will be active on the site. For "
            "example, if you enter `/records`, any URL starting with "
            "`/records` will return an active banner (`/records`, "
            "`/records/1234`, etc.). An empty value makes the banner "
            "active for any URL."
        ),
    },
    "category": {
        "order": 5,
        "text": _("Category"),
        "description": _(
            "Banner category. `Info` option displays a blue banner. "
            "`Warning` option displays an orange banner. "
            "`Other` option displays a gray banner."
        ),
        "options": [
            {"title_l10n": "Info", "id": "info"},
            {"title_l10n": "Warning", "id": "warning"},
            {"title_l10n": "Other", "id": "other"},
        ],
        "placeholder": "Select a category",
    },
    "active": {
        "order": 6,
        "text": _("Active"),
        "description": _(
            "Tick it to activate the banner: banner will be "
            "displayed according to start/end times. If not "
            "activated, start/end times will be ignored."
        ),
    },
}


class BannerEditView(AdminResourceEditView):
    """Configuration for Banner edit view."""

    name = "banner_edit"
    url = "/banners/<pid_value>/edit"
    resource_config = "banners_resource"
    pid_path = "id"
    api_endpoint = "/banners"
    title = "Edit Banner"

    list_view_name = "banners"

    form_fields = {
        **common_form_fields,
        "created": {"order": 7},
        "updated": {"order": 8},
    }


class BannerCreateView(AdminResourceCreateView):
    """Configuration for Banner create view."""

    name = "banner_create"
    url = "/banners/create"
    resource_config = "banners_resource"
    pid_path = "id"
    api_endpoint = "/banners"
    title = "Create Banner"

    list_view_name = "banners"

    form_fields = {
        **common_form_fields,
    }


class BannerDetailView(AdminResourceDetailView):
    """Admin banner detail view."""

    url = "/banners/<pid_value>"
    api_endpoint = "/banners"
    name = "banner-details"
    resource_config = "banners_resource"
    title = "Banner Details"

    display_delete = True
    display_edit = True

    list_view_name = "banners"
    pid_path = "id"

    item_field_list = {
        "start_datetime": {"text": _("Start time"), "order": 1},
        "end_datetime": {"text": _("End time"), "order": 2},
        "message": {"text": _("Message"), "order": 3},
        "url_path": {"text": _("URL path"), "order": 4},
        "category": {"text": _("Category"), "order": 5},
        "active": {"text": _("Active"), "order": 6},
        "created": {"text": _("Created"), "order": 7},
        "updated": {"text": _("Updated"), "order": 8},
    }



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 CERN.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Pytest configuration."""

import pytest
from invenio_access.models import ActionRoles
from invenio_accounts.models import Role
from invenio_administration.permissions import administration_access_action
from invenio_app.factory import create_api


@pytest.fixture(scope="module")
def celery_config():
    """Override pytest-invenio fixture."""
    return {}


@pytest.fixture(scope="module")
def create_app(instance_path):
    """Application factory fixture."""
    return create_api


@pytest.fixture()
def headers():
    """Default headers for making requests."""
    return {
        "content-type": "application/json",
        "accept": "application/json",
    }


@pytest.fixture()
def admin_role_need(db):
    """Store 1 role with 'superuser-access' ActionNeed."""
    role = Role(name="administration-access")
    db.session.add(role)

    action_role = ActionRoles.create(action=administration_access_action, role=role)
    db.session.add(action_role)

    db.session.commit()
    return action_role.need


@pytest.fixture()
def admin(UserFixture, app, db, admin_role_need):
    """Admin user for requests."""
    u = UserFixture(
        email="admin@inveniosoftware.org",
        password="admin",
    )
    u.create(app, db)

    datastore = app.extensions["security"].datastore
    _, role = datastore._prepare_role_modify_args(u.user, "administration-access")

    datastore.add_role_to_user(u.user, role)
    db.session.commit()
    return u


@pytest.fixture()
def user(UserFixture, app, db):
    """General user for requests."""
    user = UserFixture(email="user1@example.org", password="user1")
    user.create(app, db)

    return user



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Banner services conftest."""

import pytest


@pytest.fixture()
def superuser_identity(admin):
    """Superuser identity fixture."""
    identity = admin.identity
    return identity


@pytest.fixture()
def simple_user_identity(user):
    """Simple identity fixture."""
    identity = user.identity
    return identity



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 CERN.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Errors."""


class BannerNotExistsError(Exception):
    """Banner not found exception."""

    def __init__(self, banner_id):
        """Constructor."""
        self.banner_id = banner_id

    @property
    def description(self):
        """Exception's description."""
        return f"Banner with id {self.banner_id} is not found."



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2023 CERN.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Banners Service configuration."""

from invenio_i18n import gettext as _
from invenio_records_resources.services import Link, RecordServiceConfig
from invenio_records_resources.services.records.links import pagination_links
from sqlalchemy import asc, desc

from ..records.models import BannerModel
from .permissions import BannersPermissionPolicy
from .results import BannerItem, BannerList
from .schemas import BannerSchema


class BannersLink(Link):
    """Link variables setter for Banner links."""

    @staticmethod
    def vars(banner, vars):
        """Variables for the URI template."""
        vars.update({"id": banner.id})


class SearchOptions:
    """Search options."""

    sort_direction_default = "asc"
    sort_direction_options = {
        "asc": dict(
            title=_("Ascending"),
            fn=asc,
        ),
        "desc": dict(
            title=_("Descending"),
            fn=desc,
        ),
    }

    sort_default = "start_datetime"
    sort_options = {
        "url_path": dict(
            title=_("Url path"),
            fields=["url_path"],
        ),
        "start_datetime": dict(
            title=_("Start time"),
            fields=["start_datetime"],
        ),
        "end_datetime": dict(
            title=_("End time"),
            fields=["end_datetime"],
        ),
        "active": dict(
            title=_("Active"),
            fields=["active"],
        ),
    }
    pagination_options = {
        "default_results_per_page": 25,
    }


class BannerServiceConfig(RecordServiceConfig):
    """Service factory configuration."""

    result_item_cls = BannerItem
    result_list_cls = BannerList
    permission_policy_cls = BannersPermissionPolicy
    schema = BannerSchema

    # Search configuration
    search = SearchOptions

    # links configuration
    links_item = {
        "self": BannersLink("{+api}/banners/{id}"),
    }
    links_search = pagination_links("{+api}/banners{?args*}")
    record_cls = BannerModel



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 CERN.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Test disable expired."""

from datetime import datetime, timedelta

from invenio_banners.records.models import BannerModel

banners = {
    "valid": {
        "message": "valid",
        "url_path": "/valid",
        "category": "warning",
        "end_datetime": datetime.utcnow() + timedelta(days=1),
        "active": True,
    },
    "everywhere": {
        "message": "everywhere",
        "url_path": None,
        "category": "info",
        "start_datetime": datetime.utcnow() - timedelta(days=1),
        "active": True,
    },
    "sub_records_only": {
        "message": "sub_records_only",
        "url_path": "/resources/sub",
        "category": "info",
        "start_datetime": datetime.utcnow() - timedelta(days=1),
        "active": True,
    },
    "expired": {
        "message": "expired",
        "url_path": "/expired",
        "category": "info",
        "end_datetime": datetime.utcnow() - timedelta(days=1),
        "active": True,
    },
}


def test_disable_expired(app):
    """Test clean up old announcement but still active."""
    BannerModel.create(banners["everywhere"])
    BannerModel.create(banners["expired"])
    BannerModel.create(banners["valid"])
    BannerModel.create(banners["sub_records_only"])

    assert BannerModel.query.filter(BannerModel.active.is_(True)).count() == 4

    BannerModel.disable_expired()

    _banners = BannerModel.query.filter(BannerModel.active.is_(True)).all()
    assert len(_banners) == 3
    assert _banners[0].message == "everywhere"
    assert _banners[1].message == "valid"
    assert _banners[2].message == "sub_records_only"



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Banner resource conftest."""

import pytest


@pytest.fixture(scope="module")
def app_config(app_config):
    """Override pytest-invenio app_config fixture.

    For test purposes we need to enforce the configuration variables set in
    config.py. Because invenio-rdm-records is not a flavour extension, it does
    not enforce them via a config entrypoint or ext.py; only flavour
    extensions are allowed to forcefully set configuration.

    This means there is a clash between configuration set by
    invenio-records-rest and this module for instance. We want this module's
    config.py to apply in tests.
    """
    app_config["REST_CSRF_ENABLED"] = False
    return app_config


@pytest.fixture()
def headers():
    """Default headers for making requests."""
    return {
        "content-type": "application/json",
        "accept": "application/json",
    }



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2023 CERN.
# Copyright (C) 2024 Graz University of Technology.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Banner Service API."""

import distutils.util

import arrow
from invenio_db.uow import unit_of_work
from invenio_records_resources.services import RecordService
from invenio_records_resources.services.base import LinksTemplate
from invenio_records_resources.services.base.utils import map_search_params
from sqlalchemy import func

from ..records.models import BannerModel


class BannerService(RecordService):
    """Banner Service."""

    def read(self, identity, id):
        """Retrieve a banner."""
        self.require_permission(identity, "read")

        banner = self.record_cls.get(id)

        return self.result_item(
            self,
            identity,
            banner,
            links_tpl=self.links_item_tpl,
        )

    def search(self, identity, params):
        """Search for banners matching the querystring."""
        self.require_permission(identity, "search")

        search_params = map_search_params(self.config.search, params)

        query_param = search_params["q"]
        filters = []

        if query_param:
            filters.extend(
                [
                    BannerModel.url_path.ilike(f"%{query_param}%"),
                    BannerModel.message.ilike(f"%{query_param}%"),
                    BannerModel.category.ilike(f"%{query_param}%"),
                ]
            )
            bool_value = self._validate_bool(query_param)
            if bool_value is not None:
                filters.extend(
                    [
                        BannerModel.active.is_(bool_value),
                    ]
                )

            datetime_value = self._validate_datetime(query_param)
            if datetime_value is not None:
                filters.extend(
                    [
                        func.date(BannerModel.start_datetime) == datetime_value,
                        func.date(BannerModel.end_datetime) == datetime_value,
                        func.date(BannerModel.created) == datetime_value,
                        func.date(BannerModel.updated) == datetime_value,
                    ]
                )

        banners = self.record_cls.search(search_params, filters)

        return self.result_list(
            self,
            identity,
            banners,
            params=search_params,
            links_tpl=LinksTemplate(self.config.links_search, context={"args": params}),
            links_item_tpl=self.links_item_tpl,
        )

    @unit_of_work()
    def create(self, identity, data, raise_errors=True, uow=None):
        """Create a banner."""
        self.require_permission(identity, "create")

        # validate data
        valid_data, errors = self.schema.load(
            data,
            context={"identity": identity},
            raise_errors=raise_errors,
        )

        # create the banner with the specified data
        banner = self.record_cls.create(valid_data)

        return self.result_item(
            self, identity, banner, links_tpl=self.links_item_tpl, errors=errors
        )

    @unit_of_work()
    def delete(self, identity, id, uow=None):
        """Delete a banner from database."""
        self.require_permission(identity, "delete")

        banner = self.record_cls.get(id)
        self.record_cls.delete(banner)

        return self.result_item(self, identity, banner, links_tpl=self.links_item_tpl)

    @unit_of_work()
    def update(self, identity, id, data, uow=None):
        """Update a banner."""
        self.require_permission(identity, "update")

        banner = self.record_cls.get(id)

        # validate data
        valid_data, errors = self.schema.load(
            data,
            context={"identity": identity},
            raise_errors=True,
        )

        self.record_cls.update(valid_data, id)

        return self.result_item(
            self,
            identity,
            banner,
            links_tpl=self.links_item_tpl,
        )

    @unit_of_work()
    def disable_expired(self, identity, uow=None):
        """Disable expired banners."""
        self.require_permission(identity, "disable")
        self.record_cls.disable_expired()

    def _validate_bool(self, value):
        try:
            bool_value = distutils.util.strtobool(value)
        except ValueError:
            return None
        return bool(bool_value)

    def _validate_datetime(self, value):
        try:
            date_value = arrow.get(value).date()
        except ValueError:
            return None
        return date_value



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
# Copyright (C) 2024 Graz University of Technology.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Service level tests for Banners."""

from datetime import datetime, timedelta

import pytest
from invenio_db import db
from invenio_records_resources.services.errors import PermissionDeniedError

from invenio_banners.proxies import current_banners_service as service
from invenio_banners.records import BannerModel
from invenio_banners.services.errors import BannerNotExistsError

banners = {
    "active": {
        "message": "active",
        "url_path": "/active",
        "category": "info",
        "start_datetime": datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S"),
        "end_datetime": (datetime.utcnow() + timedelta(days=1)).strftime(
            "%Y-%m-%d %H:%M:%S"
        ),
        "active": True,
    },
    "inactive": {
        "message": "inactive",
        "url_path": "/inactive",
        "category": "info",
        "active": False,
    },
    "other": {
        "message": "other",
        "url_path": "/other",
        "category": "warning",
        "end_datetime": datetime.utcnow() + timedelta(days=5),
        "active": True,
    },
    "expired": {
        "message": "expired",
        "url_path": "/expired",
        "category": "info",
        "end_datetime": datetime.utcnow() - timedelta(days=1),
        "active": True,
    },
    "sub_records_only": {
        "message": "sub_records_only",
        "url_path": "/resources/sub",
        "category": "warning",
        "start_datetime": datetime.utcnow() - timedelta(days=1),
        "active": True,
    },
    "records_only": {
        "message": "records_only",
        "url_path": "/resources",
        "category": "info",
        "start_datetime": datetime.utcnow() - timedelta(days=1),
        "active": True,
    },
}


def test_banner_creation(app, superuser_identity):
    """Create a banner."""
    banner_data = banners["active"]
    banner = service.create(superuser_identity, banner_data)

    assert banner["message"] == banner_data["message"]
    assert banner["url_path"] == banner_data["url_path"]
    assert banner["category"] == banner_data["category"]
    assert banner["active"] == banner_data["active"]


def test_create_is_forbidden(app, simple_user_identity):
    """Test that the simple user cannot create a new banner."""
    with pytest.raises(PermissionDeniedError):
        service.create(simple_user_identity, banners["active"])


def test_update_banner(app, superuser_identity):
    """Update a banner."""
    # create banner first
    banner = BannerModel.create(banners["active"])

    new_data = {
        "active": True,
        "start_datetime": datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S"),
        "message": "New banner message",
        "category": "info",
    }

    updated_banner = service.update(superuser_identity, banner.id, new_data)

    assert updated_banner["message"] == new_data["message"]
    assert updated_banner["category"] == new_data["category"]
    assert updated_banner["url_path"] == banner.url_path
    assert updated_banner["active"] == banner.active


def test_update_is_forbidden(app, simple_user_identity):
    """Test that the simple user cannot update a banner."""
    # create banner first
    banner = BannerModel.create(banners["active"])

    new_data = {"message": "New banner message"}

    with pytest.raises(PermissionDeniedError):
        service.update(simple_user_identity, banner.id, new_data)


def test_update_non_existing_banner(app, superuser_identity):
    """Update a non-existing banner."""
    new_data = {"message": "New banner message"}

    with pytest.raises(BannerNotExistsError) as ex:
        service.update(superuser_identity, 1, new_data)

    assert ex.value.description == "Banner with id 1 is not found."


def test_delete_banner(app, superuser_identity):
    """Delete a banner."""
    # create banner first
    banner = BannerModel.create(banners["active"])

    service.delete(superuser_identity, banner.id)

    # check that it's not present in db
    assert db.session.query(BannerModel).filter_by(id=banner.id).one_or_none() is None


def test_delete_is_forbidden(app, simple_user_identity):
    """Test that the simple user cannot delete a banner."""
    # create banner first
    banner = BannerModel.create(banners["active"])

    with pytest.raises(PermissionDeniedError):
        service.delete(simple_user_identity, banner.id)


def test_delete_non_existing_banner(app, superuser_identity):
    """Delete a non-existing banner."""
    with pytest.raises(BannerNotExistsError) as ex:
        service.delete(superuser_identity, 1)

    assert ex.value.description == "Banner with id 1 is not found."


def test_read_banner(app, simple_user_identity):
    """Read a banner by id."""
    # create banner first
    banner = BannerModel.create(banners["other"])

    banner_result = service.read(simple_user_identity, banner.id)

    assert banner_result["message"] == banner.message
    assert banner_result["url_path"] == banner.url_path
    assert banner_result["category"] == banner.category
    assert banner_result["active"] == banner.active


def test_read_non_existing_banner(app, simple_user_identity):
    """Read a non-existing banner."""
    with pytest.raises(BannerNotExistsError) as ex:
        service.read(simple_user_identity, 1)

    assert ex.value.description == "Banner with id 1 is not found."


def test_search_banner_with_params(app, simple_user_identity):
    """Search for banners with parameters."""
    # create banners first
    BannerModel.create(banners["active"])
    BannerModel.create(banners["other"])
    BannerModel.create(banners["inactive"])
    BannerModel.create(banners["expired"])

    search_params = {
        "q": "true",
        "sort": "end_datetime",
        "size": 2,
        "sort_direction": "desc",
    }

    banner_list = service.search(simple_user_identity, params=search_params)

    assert banner_list.total == 3
    result_list = banner_list.to_dict()["hits"]["hits"]
    assert len(result_list) == 2
    assert result_list[0]["message"] == "other"
    assert result_list[1]["message"] == "active"


def test_search_banner_empty_list(app, simple_user_identity):
    """Search for banners (no banner found)."""
    banner_list = service.search(simple_user_identity, {})

    assert banner_list.total == 0
    result_list = banner_list.to_dict()["hits"]
    assert len(result_list["hits"]) == 0


def test_disable_expired_banners(app, superuser_identity):
    """Disable expired banners."""
    # create banner first
    BannerModel.create(banners["expired"])
    BannerModel.create(banners["active"])

    assert (
        db.session.query(BannerModel).filter(BannerModel.active.is_(True)).count() == 2
    )
    service.disable_expired(superuser_identity)

    _banners = db.session.query(BannerModel).filter(BannerModel.active.is_(True)).all()

    assert len(_banners) == 1
    assert _banners[0].message == "active"


def test_disable_expired_is_forbidden(app, simple_user_identity):
    """Test that the simple user cannot disable a banner."""
    with pytest.raises(PermissionDeniedError):
        service.disable_expired(simple_user_identity)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Banners permissions."""

from invenio_administration.generators import Administration
from invenio_records_permissions import BasePermissionPolicy
from invenio_records_permissions.generators import AnyUser, SystemProcess


class BannersPermissionPolicy(BasePermissionPolicy):
    """Permission policy for banners."""

    can_create = [Administration(), SystemProcess()]
    can_read = [AnyUser(), SystemProcess()]
    can_search = [AnyUser(), SystemProcess()]
    can_update = [Administration(), SystemProcess()]
    can_delete = [Administration(), SystemProcess()]
    can_disable = [Administration(), SystemProcess()]



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Banners Service API."""

from .config import BannerServiceConfig, BannersLink
from .results import BannerItem, BannerList
from .service import BannerService

__all__ = (
    "BannerService",
    "BannerServiceConfig",
    "BannerList",
    "BannerItem",
    "BannersLink",
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2023 CERN.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio Banners module to create REST APIs."""

from flask import g
from flask_resources import Resource, resource_requestctx, response_handler, route
from invenio_records_resources.resources.records.resource import (
    request_data,
    request_headers,
    request_search_args,
    request_view_args,
)

from .errors import ErrorHandlersMixin


#
# Resource
#
class BannerResource(ErrorHandlersMixin, Resource):
    """Banner resource."""

    def __init__(self, config, service):
        """Constructor."""
        super(BannerResource, self).__init__(config)
        self.service = service

    def create_url_rules(self):
        """Create the URL rules for the record resource."""
        routes = self.config.routes
        return [
            route("POST", routes["list"], self.create),
            route("GET", routes["item"], self.read),
            route("GET", routes["list"], self.search),
            route("DELETE", routes["item"], self.delete),
            route("PUT", routes["item"], self.update),
        ]

    @request_view_args
    @request_data
    @response_handler()
    def update(self):
        """Update a banner."""
        banner = self.service.update(
            id=resource_requestctx.view_args["banner_id"],
            identity=g.identity,
            data=resource_requestctx.data,
        )

        # disable expired banners
        self.service.disable_expired(identity=g.identity)

        return banner.to_dict(), 200

    @request_view_args
    @response_handler()
    def read(self):
        """Read a banner."""
        banner_id = resource_requestctx.view_args["banner_id"]
        banner = self.service.read(
            id=banner_id,
            identity=g.identity,
        )

        return banner.to_dict(), 200

    @request_search_args
    @response_handler(many=True)
    def search(self):
        """Perform a search over the banners."""
        banners = self.service.search(
            identity=g.identity,
            params=resource_requestctx.args,
        )

        return banners.to_dict(), 200

    @request_data
    @response_handler()
    def create(self):
        """Create a banner."""
        banner = self.service.create(
            g.identity,
            resource_requestctx.data or {},
        )

        # disable expired banners
        self.service.disable_expired(identity=g.identity)

        return banner.to_dict(), 201

    @request_headers
    @request_view_args
    def delete(self):
        """Delete a banner."""
        banner_id = resource_requestctx.view_args["banner_id"]
        banner = self.service.delete(
            id=banner_id,
            identity=g.identity,
        )

        return banner.to_dict(), 204



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 CERN.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio module providing management APIs for banners."""


def create_banners_api_bp(app):
    """Create the banners resource api blueprint."""
    ext = app.extensions["invenio-banners"]
    return ext.banners_resource.as_blueprint()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 CERN.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio extension app."""

from flask import Blueprint

from . import config
from .resources import BannerResource, BannerResourceConfig
from .services import BannerService, BannerServiceConfig
from .utils import get_active_banners_for_request

blueprint = Blueprint(
    "invenio_banners",
    __name__,
    template_folder="templates",
    static_folder="static",
)


class InvenioBanners(object):
    """Invenio-Banners extension."""

    def __init__(self, app=None):
        """Extension initialization."""
        if app:
            self.init_app(app)

    def init_app(self, app):
        """Flask application initialization."""
        self.init_config(app)
        self.init_services(app)
        self.init_resources(app)
        app.extensions["invenio-banners"] = self
        app.register_blueprint(blueprint)
        app.jinja_env.globals["get_active_banners"] = get_active_banners_for_request
        app.jinja_env.filters["style_banner_category"] = app.config[
            "BANNERS_CATEGORIES_TO_STYLE"
        ]

    def init_config(self, app):
        """Initialize configuration."""
        for k in dir(config):
            if k.startswith("BANNERS_"):
                app.config.setdefault(k, getattr(config, k))

    def init_services(self, app):
        """Initialize the services for banners."""
        self.banners_service = BannerService(config=BannerServiceConfig)

    def init_resources(self, app):
        """Initialize the resources for banners."""
        self.banners_resource = BannerResource(
            service=self.banners_service,
            config=BannerResourceConfig,
        )



#
# This file is part of Invenio.
# Copyright (C) 2016-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Create invenio-banners db table."""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "5e02314da32e"
down_revision = "e40d93d99040"
branch_labels = ()
depends_on = None


def upgrade():
    """Upgrade database."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "banners",
        sa.Column("created", sa.DateTime(), nullable=False),
        sa.Column("updated", sa.DateTime(), nullable=False),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("message", sa.Text(), nullable=False),
        sa.Column("url_path", sa.String(length=255), nullable=True),
        sa.Column("category", sa.String(length=20), nullable=False),
        sa.Column("start_datetime", sa.DateTime(), nullable=False),
        sa.Column("end_datetime", sa.DateTime(), nullable=True),
        sa.Column("active", sa.Boolean(name="active"), nullable=False),
        sa.PrimaryKeyConstraint("id", name=op.f("pk_banners")),
    )
    # ### end Alembic commands ###


def downgrade():
    """Downgrade database."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("banners")
    # ### end Alembic commands ###



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 CERN.
# Copyright (C) 2024 Graz University of Technology.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Models."""

from datetime import datetime

import sqlalchemy as sa
from flask import current_app
from invenio_db import db
from sqlalchemy import or_
from sqlalchemy.sql import text
from sqlalchemy_utils.models import Timestamp

from ..services.errors import BannerNotExistsError


class BannerModel(db.Model, Timestamp):
    """Defines a message to show to users."""

    __tablename__ = "banners"

    id = db.Column(db.Integer, primary_key=True)

    message = db.Column(db.Text, nullable=False)
    """The message content."""

    url_path = db.Column(db.String(255), nullable=True)
    """Define in which URL /path the message will be visible."""

    category = db.Column(db.String(20), nullable=False)
    """Category of the message, for styling messages per category."""

    start_datetime = db.Column(db.DateTime, nullable=False, default=datetime.utcnow)
    """Start date and time (UTC), can be immediate or delayed."""

    end_datetime = db.Column(db.DateTime, nullable=True)
    """End date and time (UTC), must be after `start` or forever if null."""

    active = db.Column(db.Boolean(name="active"), nullable=False, default=True)
    """Defines if the message is active, only one at the same time."""

    @classmethod
    def create(cls, data):
        """Create a new banner."""
        _categories = [t[0] for t in current_app.config["BANNERS_CATEGORIES"]]
        assert data.get("category") in _categories

        with db.session.begin_nested():
            obj = cls(
                message=data.get("message"),
                category=data.get("category"),
                url_path=data.get("url_path"),
                start_datetime=data.get("start_datetime"),
                end_datetime=data.get("end_datetime"),
                active=data.get("active"),
            )
            db.session.add(obj)

        return obj

    @classmethod
    def update(cls, data, id):
        """Update an existing banner."""
        with db.session.begin_nested():
            # NOTE:
            # with db.session.get(cls, id) the model itself would be
            # returned and this classmethod would be called
            db.session.query(cls).filter_by(id=id).update(data)

    @classmethod
    def get(cls, id):
        """Get banner by its id."""
        if banner := db.session.get(cls, id):
            return banner

        raise BannerNotExistsError(id)

    @classmethod
    def delete(cls, banner):
        """Delete banner by its id."""
        with db.session.begin_nested():
            db.session.delete(banner)

    @classmethod
    def get_active(cls, url_path):
        """Return active banners."""
        now = datetime.utcnow()

        query = (
            db.session.query(cls)
            .filter(cls.active.is_(True))
            .filter(cls.start_datetime <= now)
            .filter((cls.end_datetime.is_(None)) | (now <= cls.end_datetime))
        )

        # filter by url_path
        active_banners = query.filter(
            sa.or_(
                cls.url_path.is_(None),
                sa.literal(url_path).startswith(cls.url_path),
            )
        )

        return active_banners.all()

    @classmethod
    def search(cls, search_params, filters):
        """Filter banners accordingly to query params."""
        if filters == []:
            filtered = db.session.query(BannerModel).filter()
        else:
            filtered = db.session.query(BannerModel).filter(or_(*filters))

        banners = filtered.order_by(
            search_params["sort_direction"](text(",".join(search_params["sort"])))
        ).paginate(
            page=search_params["page"],
            per_page=search_params["size"],
            error_out=False,
        )

        return banners

    @classmethod
    def disable_expired(cls):
        """Disable any old still active messages to keep everything clean."""
        now = datetime.utcnow()

        query = (
            db.session.query(cls)
            .filter(cls.active.is_(True))
            .filter(cls.end_datetime.isnot(None))
            .filter(cls.end_datetime < now)
        )

        for old in query.all():
            old.active = False



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio Resources module to create REST APIs."""

from .config import BannerResourceConfig
from .resource import BannerResource

__all__ = (
    "BannerResource",
    "BannerResourceConfig",
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
# Copyright (C) 2024 Graz University of Technology.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Banner resource tests."""
from datetime import date, datetime, timedelta

import pytest
from invenio_db import db
from invenio_records_resources.services.errors import PermissionDeniedError

from invenio_banners.records import BannerModel

banners = {
    "banner0": {
        "message": "banner0",
        "url_path": "/banner0",
        "category": "info",
        "active": True,
        "start_datetime": date(2022, 7, 20).strftime("%Y-%m-%d %H:%M:%S"),
        "end_datetime": (datetime.utcnow() - timedelta(days=20)).strftime(
            "%Y-%m-%d %H:%M:%S"
        ),
    },
    "banner1": {
        "message": "banner1",
        "url_path": "/banner1",
        "category": "info",
        "active": True,
        "start_datetime": date(2022, 7, 20).strftime("%Y-%m-%d %H:%M:%S"),
        "end_datetime": (datetime.utcnow() + timedelta(days=20)).strftime(
            "%Y-%m-%d %H:%M:%S"
        ),
    },
    "banner2": {
        "message": "banner2",
        "url_path": "/banner2",
        "category": "other",
        "active": False,
        "start_datetime": date(2022, 12, 15).strftime("%Y-%m-%d %H:%M:%S"),
        "end_datetime": (datetime.utcnow() + timedelta(days=10)).strftime(
            "%Y-%m-%d %H:%M:%S"
        ),
    },
    "banner3": {
        "message": "banner3",
        "url_path": "/banner3",
        "category": "warning",
        "active": True,
        "start_datetime": date(2023, 1, 20).strftime("%Y-%m-%d %H:%M:%S"),
        "end_datetime": (datetime.utcnow() + timedelta(days=30)).strftime(
            "%Y-%m-%d %H:%M:%S"
        ),
    },
    "html_banner": {
        "message": '<h1>HTML aware banner.</h1>.\n \'<p class="test">paragraph<br /></p><script '
        "type=\"text/javascript\">alert('You won!')</script>",
        "url_path": "/html_banner",
        "category": "warning",
        "active": True,
        "start_datetime": datetime(2022, 7, 20, 20, 0, 0).strftime("%Y-%m-%d %H:%M:%S"),
    },
}


def _create_banner(client, data, headers, status_code=None):
    """Send POST request."""
    result = client.post("/banners/", headers=headers, json=data)
    assert result.status_code == status_code
    return result


def _update_banner(client, id, data, headers, status_code=None):
    """Send PUT request."""
    result = client.put(f"/banners/{id}", headers=headers, json=data)
    assert result.status_code == status_code
    return result


def _delete_banner(client, id, headers, status_code=None):
    """Send DELETE request."""
    result = client.delete(f"/banners/{id}", headers=headers)
    assert result.status_code == status_code
    return result


def _get_banner(client, id, status_code=None):
    """Send GET request."""
    result = client.get(f"/banners/{id}")
    assert result.status_code == status_code
    return result


def _search_banners(client, status_code=None, query_string=None):
    """Send GET request."""
    result = client.get("/banners/", query_string=query_string)
    assert result.status_code == status_code
    return result


def test_create_is_forbidden(client, user, headers):
    """Test that the simple user cannot create a new banner."""
    user.login(client)

    # try to create a banner
    with pytest.raises(PermissionDeniedError):
        _create_banner(client, banners["banner1"], headers)


def test_create_banner(client, admin, headers):
    """Create a banner."""
    banner_data = banners["banner1"]
    admin.login(client)

    banner = _create_banner(client, banner_data, headers, 201).json
    assert banner["message"] == banner_data["message"]
    assert banner["url_path"] == banner_data["url_path"]
    assert banner["category"] == banner_data["category"]
    assert banner["active"] == banner_data["active"]


def test_disable_expired_after_create_action(client, admin, headers):
    """Disable expired banners after a create a banner action."""
    # create banner first
    banner0 = BannerModel.create(banners["banner0"])
    assert banner0.active is True
    banner_data = banners["banner2"]

    admin.login(client)

    _create_banner(client, banner_data, headers, 201).json

    expired_banner = BannerModel.get(banner0.id)
    assert expired_banner.active is False


def test_update_banner(client, admin, headers):
    """Update a banner."""
    # create banner first
    banner = BannerModel.create(banners["banner1"])

    admin.login(client)

    new_data = {
        "start_datetime": datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S"),
        "active": True,
        "message": "New banner message",
        "category": "info",
    }

    updated_banner = _update_banner(client, banner.id, new_data, headers, 200).json

    assert updated_banner["message"] == new_data["message"]
    assert updated_banner["category"] == new_data["category"]
    assert updated_banner["url_path"] == banner.url_path
    assert updated_banner["active"] == banner.active


def test_disable_expired_after_update_action(client, admin, headers):
    """Disable expired banners after an update a banner action."""
    # create banner first
    banner0 = BannerModel.create(banners["banner0"])
    banner2 = BannerModel.create(banners["banner2"])
    assert banner0.active is True

    admin.login(client)

    new_data = {
        "start_datetime": datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S"),
        "active": True,
        "message": "New banner message",
        "category": "info",
    }

    _update_banner(client, banner2.id, new_data, headers, 200).json

    expired_banner = BannerModel.get(banner0.id)
    assert expired_banner.active is False


def test_update_is_forbidden(client, user, headers):
    """Test that the simple user cannot update a banner."""
    # create banner first
    banner = BannerModel.create(banners["banner1"])

    user.login(client)

    new_data = {"message": "New banner message"}

    # try to update a banner
    with pytest.raises(PermissionDeniedError):
        _update_banner(client, banner.id, new_data, headers)


def test_update_non_existing_banner(client, admin, headers):
    """Update a non-existing banner."""
    admin.login(client)

    new_data = {"message": "New banner message"}

    # try to update a banner (get 404)
    banner = _update_banner(client, 1, new_data, headers, 404).json

    assert banner["message"] == "Banner with id 1 is not found."


def test_delete_banner(client, admin, headers):
    """Delete a banner."""
    # create banner first
    banner = BannerModel.create(banners["banner1"])

    admin.login(client)

    _delete_banner(client, banner.id, headers, 204)

    # check that it's not present in db
    assert db.session.query(BannerModel).filter_by(id=banner.id).one_or_none() is None


def test_delete_is_forbidden(client, user, headers):
    """Test that the simple user cannot delete a banner."""
    # create banner first
    banner = BannerModel.create(banners["banner1"])

    user.login(client)

    # try to delete a banner
    with pytest.raises(PermissionDeniedError):
        _delete_banner(client, banner.id, headers)


def test_delete_non_existing_banner(client, admin, headers):
    """Delete a non-existing banner."""
    admin.login(client)

    # try to delete a banner (get 404)
    banner = _delete_banner(client, 1, headers, 404).json

    assert banner["message"] == "Banner with id 1 is not found."


def test_read_banner(client, user):
    """Read a banner by id."""
    # create banner first
    banner = BannerModel.create(banners["banner1"])

    user.login(client)

    banner_result = _get_banner(client, banner.id, 200).json
    assert banner_result["message"] == banner.message
    assert banner_result["url_path"] == banner.url_path
    assert banner_result["category"] == banner.category
    assert banner_result["active"] == banner.active


def test_read_non_existing_banner(client, user):
    """Read a non-existing banner."""
    user.login(client)

    # try to get a banner (get 404)
    banner = _get_banner(client, 1, 404).json

    assert banner["message"] == "Banner with id 1 is not found."


def test_search_banner(client, user):
    """Search for banners without query parameters."""
    # create banners first
    BannerModel.create(banners["banner1"])
    BannerModel.create(banners["banner2"])

    user.login(client)

    banners_result = _search_banners(client, 200).json

    result_hits = banners_result["hits"]
    assert result_hits["total"] == 2
    result_list = result_hits["hits"]
    assert len(result_list) == 2
    assert result_list[0]["message"] == "banner1"
    assert result_list[1]["message"] == "banner2"


def test_search_banner_with_params(client, user):
    """Search for banners with the query string."""
    # create banners first
    BannerModel.create(banners["banner1"])
    BannerModel.create(banners["banner2"])
    BannerModel.create(banners["banner3"])

    user.login(client)

    # filtering
    # filter by string(category)
    query_string = {"q": "warning"}
    banners_result = _search_banners(client, 200, query_string).json

    result_hits = banners_result["hits"]
    assert result_hits["total"] == 1
    assert result_hits["hits"][0]["message"] == "banner3"

    # filter by bool(active)
    query_string = {"q": "false"}
    banners_result = _search_banners(client, 200, query_string).json

    result_hits = banners_result["hits"]
    assert result_hits["total"] == 1
    assert result_hits["hits"][0]["message"] == "banner2"

    # filter by datetime(start_datetime)
    query_string = {"q": "2023-1-20"}
    banners_result = _search_banners(client, 200, query_string).json

    result_hits = banners_result["hits"]
    assert result_hits["total"] == 1
    assert result_hits["hits"][0]["message"] == "banner3"

    # sorting
    # default sort (by created field in asc order)
    banners_result = _search_banners(client, 200).json
    result_hits = banners_result["hits"]
    assert result_hits["total"] == 3
    assert result_hits["hits"][0]["message"] == "banner1"
    assert result_hits["hits"][1]["message"] == "banner2"
    assert result_hits["hits"][2]["message"] == "banner3"

    # sort by string(category)
    query_string = {"sort": "category"}
    banners_result = _search_banners(client, 200, query_string).json
    result_hits = banners_result["hits"]
    assert result_hits["total"] == 3
    assert result_hits["hits"][0]["message"] == "banner1"
    assert result_hits["hits"][1]["message"] == "banner2"
    assert result_hits["hits"][2]["message"] == "banner3"

    # sort by bool(active)
    query_string = {"sort": "active"}
    banners_result = _search_banners(client, 200, query_string).json
    result_hits = banners_result["hits"]
    assert result_hits["total"] == 3
    assert result_hits["hits"][0]["message"] == "banner2"
    assert result_hits["hits"][1]["message"] == "banner1"
    assert result_hits["hits"][2]["message"] == "banner3"

    # sort by datetime(end_datetime)
    query_string = {"sort": "end_datetime"}
    banners_result = _search_banners(client, 200, query_string).json
    result_hits = banners_result["hits"]
    assert result_hits["total"] == 3
    assert result_hits["hits"][0]["message"] == "banner2"
    assert result_hits["hits"][1]["message"] == "banner1"
    assert result_hits["hits"][2]["message"] == "banner3"

    # sorting direction
    # descending order
    query_string = {"sort_direction": "desc"}
    banners_result = _search_banners(client, 200, query_string).json
    result_hits = banners_result["hits"]
    assert result_hits["total"] == 3
    assert result_hits["hits"][0]["message"] == "banner3"
    assert result_hits["hits"][1]["message"] == "banner2"
    assert result_hits["hits"][2]["message"] == "banner1"

    # paginated search
    query_string = {"size": "2"}
    banners_result = _search_banners(client, 200, query_string).json
    result_hits = banners_result["hits"]
    assert len(result_hits) == 2
    assert result_hits["total"] == 3
    assert result_hits["hits"][0]["message"] == "banner1"
    assert result_hits["hits"][1]["message"] == "banner2"


def test_search_banner_empty_list(client, user):
    """Search for banners (no banner found)."""
    user.login(client)

    banners = _search_banners(client, 200).json

    result = banners["hits"]
    assert len(result["hits"]) == 0
    assert result["total"] == 0


def test_html_banner(client, admin, headers):
    banner_data = banners["html_banner"]
    admin.login(client)

    banner = _create_banner(client, banner_data, headers, 201).json
    assert (
        banner["message"]
        == "<h1>HTML aware banner.</h1>.\n '<p>paragraph<br></p>alert('You won!')"
    )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 CERN.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Test models."""

from datetime import datetime, timedelta

import pytest

from invenio_banners.records.models import BannerModel
from invenio_banners.services.errors import BannerNotExistsError

banners = {
    "valid": {
        "message": "valid",
        "url_path": "/valid",
        "category": "info",
        "end_datetime": datetime.utcnow() + timedelta(days=1),
        "active": True,
    },
    "everywhere": {
        "message": "everywhere",
        "url_path": None,
        "category": "info",
        "start_datetime": datetime.utcnow() - timedelta(days=1),
        "active": True,
    },
    "with_end_datetime": {
        "message": "with_end_datetime",
        "url_path": "/with_end_datetime",
        "category": "info",
        "end_datetime": datetime.utcnow() - timedelta(days=1),
        "active": True,
    },
    "records_only": {
        "message": "records_only",
        "url_path": "/resources",
        "category": "info",
        "start_datetime": datetime.utcnow() - timedelta(days=1),
        "active": True,
    },
    "sub_records_only": {
        "message": "sub_records_only",
        "url_path": "/resources/sub",
        "category": "warning",
        "start_datetime": datetime.utcnow() - timedelta(days=1),
        "active": True,
    },
    "disabled": {
        "message": "disabled",
        "url_path": "/disabled",
        "category": "info",
        "start_datetime": datetime.utcnow() - timedelta(days=1),
        "active": False,
    },
    "expired": {
        "message": "expired",
        "url_path": "/expired",
        "category": "warning",
        "end_datetime": datetime.utcnow() - timedelta(days=1),
        "active": True,
    },
}


def test_get_with_sub_path(app):
    """Test get active with sub path."""
    BannerModel.create(banners["sub_records_only"])

    assert BannerModel.get_active("/resources/sub")[0].message == "sub_records_only"
    assert BannerModel.get_active("/resources") == []
    assert BannerModel.get_active("/") == []
    assert BannerModel.get_active("/other") == []


def test_get_with_specific_path(app):
    """Test get active with specific path."""
    BannerModel.create(banners["records_only"])

    assert BannerModel.get_active("/resources")[0].message == "records_only"
    assert BannerModel.get_active("/resources/other")[0].message == "records_only"
    assert BannerModel.get_active("/") == []
    assert BannerModel.get_active("/other") == []


def test_get_active_with_datetime(app):
    """Test get active with end_datetime in past."""
    BannerModel.create(banners["with_end_datetime"])
    assert BannerModel.get_active("/with_end_datetime") == []


def test_get_active_future_datetime(app, db):
    """Test get active with future date."""
    BannerModel.create(banners["expired"])
    assert BannerModel.get_active("/expired") == []


def test_get_active_disabled(app, db):
    """Test get non-active."""
    BannerModel.create(banners["disabled"])
    assert BannerModel.get_active("/disabled") == []


def test_create_banner(app):
    """Create a banner."""
    banner = BannerModel.create(banners["valid"])

    db_banner = BannerModel.query.filter_by(id=banner.id).one()
    assert db_banner.message == banner.message
    assert db_banner.category == banner.category
    assert db_banner.url_path == banner.url_path
    assert db_banner.active == banner.active


def test_update_banner(app):
    """Update a banner."""
    # create banner first
    banner = BannerModel.create(banners["valid"])

    new_data = {
        "message": "New banner message",
        "category": "other",
    }

    BannerModel.update(new_data, banner.id)

    db_banner = BannerModel.query.filter_by(id=banner.id).one()
    assert db_banner.message == new_data["message"]
    assert db_banner.category == new_data["category"]
    assert db_banner.url_path == banner.url_path
    assert db_banner.active == banner.active


def test_delete_banner(app):
    """Delete a banner."""
    # create banner first
    banner = BannerModel.create(banners["valid"])

    BannerModel.delete(banner)

    # check that it's not present in db
    assert BannerModel.query.filter_by(id=banner.id).one_or_none() is None


def test_get_banner(app):
    """Get a banner by id."""
    # create banner first
    banner = BannerModel.create(banners["valid"])

    banner_result = BannerModel.get(banner.id)
    assert banner_result.message == banner.message
    assert banner_result.category == banner.category
    assert banner_result.url_path == banner.url_path
    assert banner_result.active == banner.active


def test_get_non_existing_banner(app):
    """Get a non-existing banner."""
    # create banner first
    with pytest.raises(BannerNotExistsError) as ex:
        BannerModel.get("000")

    assert ex.value.description == "Banner with id 000 is not found."


def test_category(app):
    """Test create banner with incorrect category."""
    banner = BannerModel.get(1)
    assert banner.message == "sub_records_only"
    assert banner.category == "warning"

    with pytest.raises(AssertionError):
        banners["sub_records_only"]["category"] = "wrong"
        BannerModel.create(banners["sub_records_only"])



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 CERN.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Configuration variables."""

from invenio_i18n import lazy_gettext as _

from invenio_banners.utils import style_category

BANNERS_CATEGORIES = [
    ("info", "Info"),
    ("warning", "Warning"),
    ("other", "Other"),
]
"""Categories to define different types of messages. List of (id, label)."""

BANNERS_CATEGORIES_TO_STYLE = style_category
"""Function to transform the banner category to a specific Semantic-UI class."""

BANNERS_SEARCH = {
    "facets": [],
    "sort": [
        "url_path",
        "start_datetime",
        "end_datetime",
        "active",
    ],
}
"""Banner search configuration (i.e list of banners)"""

BANNERS_SORT_OPTIONS = {
    "url_path": dict(
        title=_("URL path"),
        fields=["url_path"],
    ),
    "start_datetime": dict(
        title=_("Start DateTime"),
        fields=["start_datetime"],
    ),
    "end_datetime": dict(
        title=_("End DateTime"),
        fields=["end_datetime"],
    ),
    "active": dict(
        title=_("Active"),
        fields=["active"],
    ),
}
"""Definitions of available Banners sort options. """



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 CERN.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Test views."""

from datetime import datetime

import pytest
from flask import url_for

from invenio_banners.records.models import BannerModel
from invenio_banners.utils import style_category


def _create_banner(message, category, url_path=None):
    """Util to create a banner."""
    banner = BannerModel.create(
        {
            "message": message,
            "category": category,
            "url_path": url_path,
            "start_datetime": datetime.utcnow(),
            "active": True,
        }
    )

    return banner


def test_jinja_macro(app, db):
    """Test the rendering of the banner in a template via macro."""

    tpl = """
    <html>
        <body>
            {%- from "semantic-ui/invenio_banners/banner.html" import banner -%}
            {{ banner() }}
        </body>
    </html>
    """
    template = app.jinja_env.from_string(tpl)
    # should be empty, no banners
    html = template.render()
    one_line = html.replace("\n", "").replace(" ", "")
    assert one_line == "<html><body></body></html>"

    # add one banner
    EXPECTED_MSG = "Test banner info message"
    EXPECTED_CATEGORY = "info"
    EXPECTED_STYLE = "ui info message top attached m-0 inv-banner"
    banner = _create_banner(EXPECTED_MSG, EXPECTED_CATEGORY)
    style = style_category(banner.category)
    html = template.render()
    assert EXPECTED_MSG in html
    assert style.endswith(EXPECTED_CATEGORY)
    assert EXPECTED_STYLE in html

    # change message and category
    EXPECTED_MSG = "Test banner warning message"
    EXPECTED_CATEGORY = "warning"
    EXPECTED_STYLE = "ui warning message top attached m-0 inv-banner"

    banner.message = EXPECTED_MSG
    banner.category = EXPECTED_CATEGORY
    db.session.commit()

    style = style_category(banner.category)

    html = template.render()
    assert EXPECTED_MSG in html
    assert style.endswith(EXPECTED_CATEGORY)
    assert EXPECTED_STYLE in html



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 CERN.
#
# Invenio-Banners is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Create and show banners with useful messages to users."""

from setuptools import setup

setup()
